package api

import (
	"encoding/json"
	"log"
	"net/http"
	"time"

	"gorm.io/gorm"
)

type Topup struct {
	gorm.Model    `json:"-"`
	TransactionID uint   `json:"-" gorm:"column:transaction"`
	MemberNum     int    `json:"member" gorm:"column:member"`
	Member        Member `json:"-" gorm:"foreignKey:MemberNum;references:Num"`
	Comment       string `json:"comment"`
}

func (a *api) AddTopup(adminNum int, w http.ResponseWriter, req *http.Request) {
	var topup struct {
		Member  int    `json:"member"`
		Comment string `json:"comment"`
		Amount  int    `json:"amount"`
	}
	err := json.NewDecoder(req.Body).Decode(&topup)
	if err != nil {
		log.Printf("Can't parse topup: %v", err)
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

	transaction := Transaction{
		MemberNum: topup.Member,
		Date:      time.Now(),
		Topup: &Topup{
			MemberNum: adminNum,
			Comment:   topup.Comment,
		},
		Type:  "topup",
		Total: topup.Amount,
	}
	httpStatus, err := createTransaction(a.db, &transaction)
	if err != nil {
		log.Println(err)
		w.WriteHeader(httpStatus)
		return
	}

	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusCreated)
	err = json.NewEncoder(w).Encode(transaction)
	if err != nil {
		log.Printf("Can't encode added topup: %v", err)
		w.WriteHeader(http.StatusInternalServerError)
	}

}
